# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from cvat_sdk.api_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from cvat_sdk.api_client.model.about import About, IAbout
from cvat_sdk.api_client.model.accept_invitation_read import AcceptInvitationRead, IAcceptInvitationRead
from cvat_sdk.api_client.model.annotation_conflict import AnnotationConflict, IAnnotationConflict
from cvat_sdk.api_client.model.annotation_conflict_annotation_type import AnnotationConflictAnnotationType, IAnnotationConflictAnnotationType
from cvat_sdk.api_client.model.annotation_conflict_severity import AnnotationConflictSeverity, IAnnotationConflictSeverity
from cvat_sdk.api_client.model.annotation_conflict_type import AnnotationConflictType, IAnnotationConflictType
from cvat_sdk.api_client.model.annotation_file_request import AnnotationFileRequest, IAnnotationFileRequest
from cvat_sdk.api_client.model.annotation_guide_read import AnnotationGuideRead, IAnnotationGuideRead
from cvat_sdk.api_client.model.annotation_guide_write_request import AnnotationGuideWriteRequest, IAnnotationGuideWriteRequest
from cvat_sdk.api_client.model.annotation_id import AnnotationId, IAnnotationId
from cvat_sdk.api_client.model.annotation_id_shape_type import AnnotationIdShapeType
from cvat_sdk.api_client.model.asset_read import AssetRead, IAssetRead
from cvat_sdk.api_client.model.attribute import Attribute, IAttribute
from cvat_sdk.api_client.model.attribute_request import AttributeRequest, IAttributeRequest
from cvat_sdk.api_client.model.attribute_val import AttributeVal, IAttributeVal
from cvat_sdk.api_client.model.attribute_val_request import AttributeValRequest, IAttributeValRequest
from cvat_sdk.api_client.model.basic_organization import BasicOrganization, IBasicOrganization
from cvat_sdk.api_client.model.basic_user import BasicUser, IBasicUser
from cvat_sdk.api_client.model.basic_user_request import BasicUserRequest, IBasicUserRequest
from cvat_sdk.api_client.model.chunk_type import ChunkType, IChunkType
from cvat_sdk.api_client.model.client_events import ClientEvents, IClientEvents
from cvat_sdk.api_client.model.client_events_request import ClientEventsRequest, IClientEventsRequest
from cvat_sdk.api_client.model.client_events_request_previous_event import ClientEventsRequestPreviousEvent
from cvat_sdk.api_client.model.cloud_storage_content import CloudStorageContent, ICloudStorageContent
from cvat_sdk.api_client.model.cloud_storage_read import CloudStorageRead, ICloudStorageRead
from cvat_sdk.api_client.model.cloud_storage_read_owner import CloudStorageReadOwner
from cvat_sdk.api_client.model.cloud_storage_write_request import CloudStorageWriteRequest, ICloudStorageWriteRequest
from cvat_sdk.api_client.model.comment_read import CommentRead, ICommentRead
from cvat_sdk.api_client.model.comment_write_request import CommentWriteRequest, ICommentWriteRequest
from cvat_sdk.api_client.model.comments_summary import CommentsSummary, ICommentsSummary
from cvat_sdk.api_client.model.consensus_merge_create_request import ConsensusMergeCreateRequest, IConsensusMergeCreateRequest
from cvat_sdk.api_client.model.consensus_settings import ConsensusSettings, IConsensusSettings
from cvat_sdk.api_client.model.credentials_type_enum import CredentialsTypeEnum, ICredentialsTypeEnum
from cvat_sdk.api_client.model.data_meta_read import DataMetaRead, IDataMetaRead
from cvat_sdk.api_client.model.data_request import DataRequest, IDataRequest
from cvat_sdk.api_client.model.data_request_validation_params import DataRequestValidationParams
from cvat_sdk.api_client.model.data_response import DataResponse
from cvat_sdk.api_client.model.dataset_file_request import DatasetFileRequest, IDatasetFileRequest
from cvat_sdk.api_client.model.dataset_format import DatasetFormat, IDatasetFormat
from cvat_sdk.api_client.model.dataset_formats import DatasetFormats, IDatasetFormats
from cvat_sdk.api_client.model.event import Event, IEvent
from cvat_sdk.api_client.model.event_request import EventRequest, IEventRequest
from cvat_sdk.api_client.model.events import Events, IEvents
from cvat_sdk.api_client.model.events_enum import EventsEnum, IEventsEnum
from cvat_sdk.api_client.model.file_info import FileInfo, IFileInfo
from cvat_sdk.api_client.model.file_info_type_enum import FileInfoTypeEnum, IFileInfoTypeEnum
from cvat_sdk.api_client.model.frame_meta import FrameMeta, IFrameMeta
from cvat_sdk.api_client.model.frame_selection_method import FrameSelectionMethod, IFrameSelectionMethod
from cvat_sdk.api_client.model.function_call import FunctionCall, IFunctionCall
from cvat_sdk.api_client.model.function_call_params import FunctionCallParams, IFunctionCallParams
from cvat_sdk.api_client.model.function_call_request import FunctionCallRequest, IFunctionCallRequest
from cvat_sdk.api_client.model.input_type_enum import InputTypeEnum, IInputTypeEnum
from cvat_sdk.api_client.model.invitation_read import InvitationRead, IInvitationRead
from cvat_sdk.api_client.model.invitation_write_request import InvitationWriteRequest, IInvitationWriteRequest
from cvat_sdk.api_client.model.issue_read import IssueRead, IIssueRead
from cvat_sdk.api_client.model.issue_write_request import IssueWriteRequest, IIssueWriteRequest
from cvat_sdk.api_client.model.issues_summary import IssuesSummary, IIssuesSummary
from cvat_sdk.api_client.model.job_read import JobRead, IJobRead
from cvat_sdk.api_client.model.job_read_assignee import JobReadAssignee
from cvat_sdk.api_client.model.job_read_target_storage import JobReadTargetStorage
from cvat_sdk.api_client.model.job_stage import JobStage, IJobStage
from cvat_sdk.api_client.model.job_status import JobStatus, IJobStatus
from cvat_sdk.api_client.model.job_type import JobType, IJobType
from cvat_sdk.api_client.model.job_validation_layout_read import JobValidationLayoutRead, IJobValidationLayoutRead
from cvat_sdk.api_client.model.job_write_request import JobWriteRequest, IJobWriteRequest
from cvat_sdk.api_client.model.jobs_summary import JobsSummary, IJobsSummary
from cvat_sdk.api_client.model.label import Label, ILabel
from cvat_sdk.api_client.model.label_mapping_entry_request import LabelMappingEntryRequest, ILabelMappingEntryRequest
from cvat_sdk.api_client.model.label_type import LabelType, ILabelType
from cvat_sdk.api_client.model.labeled_data import LabeledData, ILabeledData
from cvat_sdk.api_client.model.labeled_data_request import LabeledDataRequest, ILabeledDataRequest
from cvat_sdk.api_client.model.labeled_image import LabeledImage, ILabeledImage
from cvat_sdk.api_client.model.labeled_image_request import LabeledImageRequest, ILabeledImageRequest
from cvat_sdk.api_client.model.labeled_shape import LabeledShape, ILabeledShape
from cvat_sdk.api_client.model.labeled_shape_request import LabeledShapeRequest, ILabeledShapeRequest
from cvat_sdk.api_client.model.labeled_track import LabeledTrack, ILabeledTrack
from cvat_sdk.api_client.model.labeled_track_request import LabeledTrackRequest, ILabeledTrackRequest
from cvat_sdk.api_client.model.labels_summary import LabelsSummary, ILabelsSummary
from cvat_sdk.api_client.model.location_enum import LocationEnum, ILocationEnum
from cvat_sdk.api_client.model.login_serializer_ex_request import LoginSerializerExRequest, ILoginSerializerExRequest
from cvat_sdk.api_client.model.membership_read import MembershipRead, IMembershipRead
from cvat_sdk.api_client.model.meta_user import MetaUser
from cvat_sdk.api_client.model.null_enum import NullEnum, INullEnum
from cvat_sdk.api_client.model.online_function_call_request import OnlineFunctionCallRequest, IOnlineFunctionCallRequest
from cvat_sdk.api_client.model.operation_status import OperationStatus, IOperationStatus
from cvat_sdk.api_client.model.organization_read import OrganizationRead, IOrganizationRead
from cvat_sdk.api_client.model.organization_write_request import OrganizationWriteRequest, IOrganizationWriteRequest
from cvat_sdk.api_client.model.paginated_annotation_conflict_list import PaginatedAnnotationConflictList, IPaginatedAnnotationConflictList
from cvat_sdk.api_client.model.paginated_cloud_storage_read_list import PaginatedCloudStorageReadList, IPaginatedCloudStorageReadList
from cvat_sdk.api_client.model.paginated_comment_read_list import PaginatedCommentReadList, IPaginatedCommentReadList
from cvat_sdk.api_client.model.paginated_consensus_settings_list import PaginatedConsensusSettingsList, IPaginatedConsensusSettingsList
from cvat_sdk.api_client.model.paginated_invitation_read_list import PaginatedInvitationReadList, IPaginatedInvitationReadList
from cvat_sdk.api_client.model.paginated_issue_read_list import PaginatedIssueReadList, IPaginatedIssueReadList
from cvat_sdk.api_client.model.paginated_job_read_list import PaginatedJobReadList, IPaginatedJobReadList
from cvat_sdk.api_client.model.paginated_label_list import PaginatedLabelList, IPaginatedLabelList
from cvat_sdk.api_client.model.paginated_membership_read_list import PaginatedMembershipReadList, IPaginatedMembershipReadList
from cvat_sdk.api_client.model.paginated_meta_user_list import PaginatedMetaUserList, IPaginatedMetaUserList
from cvat_sdk.api_client.model.paginated_organization_read_list import PaginatedOrganizationReadList, IPaginatedOrganizationReadList
from cvat_sdk.api_client.model.paginated_project_read_list import PaginatedProjectReadList, IPaginatedProjectReadList
from cvat_sdk.api_client.model.paginated_quality_report_list import PaginatedQualityReportList, IPaginatedQualityReportList
from cvat_sdk.api_client.model.paginated_quality_settings_list import PaginatedQualitySettingsList, IPaginatedQualitySettingsList
from cvat_sdk.api_client.model.paginated_request_list import PaginatedRequestList, IPaginatedRequestList
from cvat_sdk.api_client.model.paginated_task_read_list import PaginatedTaskReadList, IPaginatedTaskReadList
from cvat_sdk.api_client.model.paginated_webhook_delivery_read_list import PaginatedWebhookDeliveryReadList, IPaginatedWebhookDeliveryReadList
from cvat_sdk.api_client.model.paginated_webhook_read_list import PaginatedWebhookReadList, IPaginatedWebhookReadList
from cvat_sdk.api_client.model.password_change_request import PasswordChangeRequest, IPasswordChangeRequest
from cvat_sdk.api_client.model.password_reset_confirm_request import PasswordResetConfirmRequest, IPasswordResetConfirmRequest
from cvat_sdk.api_client.model.password_reset_serializer_ex_request import PasswordResetSerializerExRequest, IPasswordResetSerializerExRequest
from cvat_sdk.api_client.model.patched_annotation_guide_write_request import PatchedAnnotationGuideWriteRequest, IPatchedAnnotationGuideWriteRequest
from cvat_sdk.api_client.model.patched_cloud_storage_write_request import PatchedCloudStorageWriteRequest, IPatchedCloudStorageWriteRequest
from cvat_sdk.api_client.model.patched_comment_write_request import PatchedCommentWriteRequest, IPatchedCommentWriteRequest
from cvat_sdk.api_client.model.patched_consensus_settings_request import PatchedConsensusSettingsRequest, IPatchedConsensusSettingsRequest
from cvat_sdk.api_client.model.patched_data_meta_write_request import PatchedDataMetaWriteRequest, IPatchedDataMetaWriteRequest
from cvat_sdk.api_client.model.patched_invitation_write_request import PatchedInvitationWriteRequest, IPatchedInvitationWriteRequest
from cvat_sdk.api_client.model.patched_issue_write_request import PatchedIssueWriteRequest, IPatchedIssueWriteRequest
from cvat_sdk.api_client.model.patched_job_data_meta_write_request import PatchedJobDataMetaWriteRequest, IPatchedJobDataMetaWriteRequest
from cvat_sdk.api_client.model.patched_job_validation_layout_write_request import PatchedJobValidationLayoutWriteRequest, IPatchedJobValidationLayoutWriteRequest
from cvat_sdk.api_client.model.patched_job_write_request import PatchedJobWriteRequest, IPatchedJobWriteRequest
from cvat_sdk.api_client.model.patched_label_request import PatchedLabelRequest, IPatchedLabelRequest
from cvat_sdk.api_client.model.patched_labeled_data_request import PatchedLabeledDataRequest, IPatchedLabeledDataRequest
from cvat_sdk.api_client.model.patched_membership_write_request import PatchedMembershipWriteRequest, IPatchedMembershipWriteRequest
from cvat_sdk.api_client.model.patched_organization_write_request import PatchedOrganizationWriteRequest, IPatchedOrganizationWriteRequest
from cvat_sdk.api_client.model.patched_project_write_request import PatchedProjectWriteRequest, IPatchedProjectWriteRequest
from cvat_sdk.api_client.model.patched_project_write_request_target_storage import PatchedProjectWriteRequestTargetStorage
from cvat_sdk.api_client.model.patched_quality_settings_request import PatchedQualitySettingsRequest, IPatchedQualitySettingsRequest
from cvat_sdk.api_client.model.patched_task_validation_layout_write_request import PatchedTaskValidationLayoutWriteRequest, IPatchedTaskValidationLayoutWriteRequest
from cvat_sdk.api_client.model.patched_task_write_request import PatchedTaskWriteRequest, IPatchedTaskWriteRequest
from cvat_sdk.api_client.model.patched_task_write_request_target_storage import PatchedTaskWriteRequestTargetStorage
from cvat_sdk.api_client.model.patched_user_request import PatchedUserRequest, IPatchedUserRequest
from cvat_sdk.api_client.model.patched_webhook_write_request import PatchedWebhookWriteRequest, IPatchedWebhookWriteRequest
from cvat_sdk.api_client.model.plugins import Plugins, IPlugins
from cvat_sdk.api_client.model.project_file_request import ProjectFileRequest, IProjectFileRequest
from cvat_sdk.api_client.model.project_read import ProjectRead, IProjectRead
from cvat_sdk.api_client.model.project_read_target_storage import ProjectReadTargetStorage
from cvat_sdk.api_client.model.project_write_request import ProjectWriteRequest, IProjectWriteRequest
from cvat_sdk.api_client.model.provider_type_enum import ProviderTypeEnum, IProviderTypeEnum
from cvat_sdk.api_client.model.quality_point_size_base import QualityPointSizeBase, IQualityPointSizeBase
from cvat_sdk.api_client.model.quality_report import QualityReport, IQualityReport
from cvat_sdk.api_client.model.quality_report_create_request import QualityReportCreateRequest, IQualityReportCreateRequest
from cvat_sdk.api_client.model.quality_report_jobs_summary import QualityReportJobsSummary, IQualityReportJobsSummary
from cvat_sdk.api_client.model.quality_report_summary import QualityReportSummary, IQualityReportSummary
from cvat_sdk.api_client.model.quality_report_summary_jobs import QualityReportSummaryJobs
from cvat_sdk.api_client.model.quality_report_summary_tasks import QualityReportSummaryTasks
from cvat_sdk.api_client.model.quality_report_target import QualityReportTarget, IQualityReportTarget
from cvat_sdk.api_client.model.quality_report_tasks_summary import QualityReportTasksSummary, IQualityReportTasksSummary
from cvat_sdk.api_client.model.quality_settings import QualitySettings, IQualitySettings
from cvat_sdk.api_client.model.quality_target_metric import QualityTargetMetric, IQualityTargetMetric
from cvat_sdk.api_client.model.register_serializer_ex import RegisterSerializerEx, IRegisterSerializerEx
from cvat_sdk.api_client.model.register_serializer_ex_request import RegisterSerializerExRequest, IRegisterSerializerExRequest
from cvat_sdk.api_client.model.request import Request, IRequest
from cvat_sdk.api_client.model.request_data_operation import RequestDataOperation, IRequestDataOperation
from cvat_sdk.api_client.model.request_owner import RequestOwner
from cvat_sdk.api_client.model.request_status import RequestStatus, IRequestStatus
from cvat_sdk.api_client.model.rest_auth_detail import RestAuthDetail, IRestAuthDetail
from cvat_sdk.api_client.model.role_enum import RoleEnum, IRoleEnum
from cvat_sdk.api_client.model.rq_id import RqId, IRqId
from cvat_sdk.api_client.model.rq_status import RqStatus, IRqStatus
from cvat_sdk.api_client.model.rq_status_state_enum import RqStatusStateEnum, IRqStatusStateEnum
from cvat_sdk.api_client.model.shape_type import ShapeType, IShapeType
from cvat_sdk.api_client.model.signing_request import SigningRequest, ISigningRequest
from cvat_sdk.api_client.model.sorting_method import SortingMethod, ISortingMethod
from cvat_sdk.api_client.model.storage import Storage, IStorage
from cvat_sdk.api_client.model.storage_method import StorageMethod, IStorageMethod
from cvat_sdk.api_client.model.storage_request import StorageRequest, IStorageRequest
from cvat_sdk.api_client.model.storage_type import StorageType, IStorageType
from cvat_sdk.api_client.model.sub_labeled_shape import SubLabeledShape, ISubLabeledShape
from cvat_sdk.api_client.model.sub_labeled_shape_request import SubLabeledShapeRequest, ISubLabeledShapeRequest
from cvat_sdk.api_client.model.sub_labeled_track import SubLabeledTrack, ISubLabeledTrack
from cvat_sdk.api_client.model.sub_labeled_track_request import SubLabeledTrackRequest, ISubLabeledTrackRequest
from cvat_sdk.api_client.model.sublabel import Sublabel, ISublabel
from cvat_sdk.api_client.model.sublabel_mapping_entry_request import SublabelMappingEntryRequest, ISublabelMappingEntryRequest
from cvat_sdk.api_client.model.sublabel_request import SublabelRequest, ISublabelRequest
from cvat_sdk.api_client.model.task_file_request import TaskFileRequest, ITaskFileRequest
from cvat_sdk.api_client.model.task_read import TaskRead, ITaskRead
from cvat_sdk.api_client.model.task_validation_layout_read import TaskValidationLayoutRead, ITaskValidationLayoutRead
from cvat_sdk.api_client.model.task_validation_layout_read_mode import TaskValidationLayoutReadMode
from cvat_sdk.api_client.model.task_write_request import TaskWriteRequest, ITaskWriteRequest
from cvat_sdk.api_client.model.tasks_summary import TasksSummary, ITasksSummary
from cvat_sdk.api_client.model.token import Token, IToken
from cvat_sdk.api_client.model.tracked_shape import TrackedShape, ITrackedShape
from cvat_sdk.api_client.model.tracked_shape_request import TrackedShapeRequest, ITrackedShapeRequest
from cvat_sdk.api_client.model.user import User, IUser
from cvat_sdk.api_client.model.user_identifiers import UserIdentifiers, IUserIdentifiers
from cvat_sdk.api_client.model.validation_mode import ValidationMode, IValidationMode
from cvat_sdk.api_client.model.validation_params_request import ValidationParamsRequest, IValidationParamsRequest
from cvat_sdk.api_client.model.webhook_content_type import WebhookContentType, IWebhookContentType
from cvat_sdk.api_client.model.webhook_delivery_read import WebhookDeliveryRead, IWebhookDeliveryRead
from cvat_sdk.api_client.model.webhook_read import WebhookRead, IWebhookRead
from cvat_sdk.api_client.model.webhook_type import WebhookType, IWebhookType
from cvat_sdk.api_client.model.webhook_write_request import WebhookWriteRequest, IWebhookWriteRequest
