# Copyright (C) 2022 CVAT.ai Corporation
#
# SPDX-License-Identifier: MIT

# CVAT REST API
#
# REST API for Computer Vision Annotation Tool (CVAT)  # noqa: E501
#
# The version of the OpenAPI document: 2.40.1
# Contact: support@cvat.ai
# Generated by: https://openapi-generator.tech


__version__ = "2.40.1"

from cvat_sdk.api_client.api_client import ApiClient

from cvat_sdk.api_client.configuration import Configuration

from cvat_sdk.api_client.exceptions import OpenApiException
from cvat_sdk.api_client.exceptions import ApiAttributeError
from cvat_sdk.api_client.exceptions import ApiTypeError
from cvat_sdk.api_client.exceptions import ApiValueError
from cvat_sdk.api_client.exceptions import ApiKeyError
from cvat_sdk.api_client.exceptions import ApiException
